#version 450 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 texCoordinate;
layout(location = 3) in ivec4 boneIds;
layout(location = 4) in vec4 boneWeights;

out vec3 fragPos;
out vec3 fragNormal;
out vec2 fragTexCoordinate;

uniform mat4 viewProjectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 bones[100];

void main() {
    vec4 pos = modelMatrix * vec4(position, 1.0);

    mat4 boneTransform = mat4(1.0f);
    if (boneWeights[0] > 0.0f) {
        boneTransform = bones[boneIds[0]] * boneWeights[0];
        boneTransform += bones[boneIds[1]] * boneWeights[1];
        boneTransform += bones[boneIds[2]] * boneWeights[2];
        boneTransform += bones[boneIds[3]] * boneWeights[3];
        pos = boneTransform * vec4(position, 1.0);
    }

    vec4 vertexPos = pos;
    gl_Position = viewProjectionMatrix * vertexPos;

    fragPos = vec3(vertexPos);
    fragNormal = mat3(transpose(inverse(modelMatrix))) * mat3(boneTransform) * normal;
    fragTexCoordinate = texCoordinate;
}